#ifndef CONFIG_H_
#define CONFIG_H_

#define DEBUG			// pokud je definovn debug, pelo i debugovac funkce

#define F_CPU 16000000UL
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>


typedef enum { FALSE, TRUE } boolean_t;
typedef enum { PROGRAM,		// sputn programu
			   USER,		// run ovldn
			   REFERENCE,	// hledn referenho bodu
			   PC_WAIT,		// ekn na odpovd potae
			   ERROR		// nkde nco v erroru
			 } mode_t;
typedef enum { ITWORKS,		// ve v podku
			   GENERIC_ERR,	// obecn chyba
			   EEPROM_ERR,	// problm s eepromkou
			   ADC_ERR,		// problm s ADC pevodnkem
			   LOAD_ERR,	// peten motor
			   TEMP_ERR,	// peht budie
			   UART_ERR,	// problm se sriovou linkou
			   PC_ACK		// bez odpovdi PC
			 } errorstate_t;
			 

			 
#define CNT_OF_ERR	 7		 

	// toto potom pjde petdit jednotlivejm hlavikovejm souborm
extern const char FIRM_INFO[];
extern const char EEP_FORMAT[];
extern const char EEP_WRITE[];
extern const char EEP_PRGNAME[];
extern const char EEP_COORD[];
extern const char INVALID[];
extern const char GENERIC_ERROR[];
extern const char HELLOWORLD[];
extern const char NEWLINE[];

// --------------------------------------------- //
//        PINMAPPING							 //
//                                               //
// --- namapovn pinu procesoru                 //
// --------------------------------------------- //
	/* motory
	DC1_A - PA7
	DC1_B - PA6
	DC2_A - PA5
	DC2_B - PA4
	DC3_A - PC4
	DC3_B - PC5
	DC4_A - PC6
	DC4_B - PC7
	DC5_A - PC0
	DC5_B - PC1
	DC6_A - PC2
	DC6_B - PC3

	*/

#define WDT_ENABLE wdt_enable (WDTO_2S);
#define WDT_DISABLE wdt_disable();
#define WDT_RESET wdt_reset();


#define DMX_PORT PORTD 
#define DMX_DDR  DDRD 
#define DMX_PIN  PIND
#define DMX_A	 PD5
#define DMX_B	 PD6
#define DMX_C	 PD7




	#define ERR_LED_PIN PIN0
	#define ERR_LED_PORT PORTD			
	#define ERR_LED_DDR  DDRD
		
	#define ERR_LED_SETUP  ERR_LED_DDR |=  (1 << ERR_LED_PIN);	
	#define ERR_LED_ON  ERR_LED_PORT |=  (1 << ERR_LED_PIN); 
	#define ERR_LED_OFF ERR_LED_PORT  &= ~(1 << ERR_LED_PIN);
	#define ERR_LED_NEG ERR_LED_PORT  ^= (1 << ERR_LED_PIN);

// --------------------------------------------- //
//        NASTAVEN DEBUGOVAC FUNKC a MAKER    //
//                                               //
// --- definice propojen LED s mcu              //
// --------------------------------------------- //
#ifdef DEBUG
	

// --- definice makro pro komunikaci             //
// --------------------------------------------- //
	#define SEND  uart_putstr_f(HELLOWORLD);
#endif
// _____________________________________________ //


// --------------------------------------------- //
//        NASTAVEN PRO ADC			             //
//                                               //
// --- definice propojen pevodnku             //
// --------------------------------------------- //
	#define DC1_MEASURE  PIN0	      
	#define DC2_MEASURE  PIN1	      
	#define DC3_MEASURE  PIN2	    
 	#define DC4_MEASURE  PIN3	      
 	#define DC5_MEASURE  PIN4      
 	#define DC6_MEASURE  PIN5
 	#define PWR_MEASURE  PIN6
	#define TEMP_MEASURE  PIN7
	
	#define ADC_DDR   DDRF	
	#define ADC_PORT  PORTF

// _____________________________________________ //

#define FAN_DDR    DDRB
#define FAN_PORT   PORTB
#define FAN_PIN    PB4

#define FAN_SETUP  FAN_DDR |=  (1 <<  FAN_PIN);
#define FAN_TEST   FAN_PORT |=  (1 <<  FAN_PIN); _delay_ms(255); FAN_PORT &= ~(1 <<  FAN_PIN)
 



// --------------------------------------------- //
//        NASTAVEN SRIOVHO KANLU             //
//                                               //
// --- definice pinu UART                        //
// --------------------------------------------- //
	#define RX_PIN  PIN0	 
	#define TX_PIN  PIN1
	#define RX_LED  PIN2
	#define TX_LED  PIN3
	  
	#define RX_DDR  DDRE	
	#define TX_DDR  DDRE
	#define RX_LED_DDR  DDRD	
	#define TX_LED_DDR  DDRD
	#define RX_TX_PORT  PORTD
	// --------------------------------------------- //
	// nastaveni serioveho kanalu
	#define TXBUFFLEN 33
	#define RXBUFFLEN 64
	#define NUMOFPACKET 5
	#define USART_BAUDRATE  19200  //  skutench 38461, chyba 0.1588%
	#define BAUD_PRESCALE (((F_CPU / ((USART_BAUDRATE / 2) * 16UL))) - 1)
	// --------------------------------------------- //
	// nastaven pomocnch funck
	#define RX_LED_ON  RX_TX_PORT |=  (1 << RX_LED)
	#define RX_LED_OFF RX_TX_PORT  &= ~(1 << RX_LED)
	#define TX_LED_ON  RX_TX_PORT |=  (1 << TX_LED)
	#define TX_LED_OFF RX_TX_PORT  &= ~(1 << TX_LED)
// _____________________________________________ //

#endif /* CONFIG_H_ */
