#include <avr/eeprom.h>
#include <string.h>
#include "../config.h"
#include "data_transfer.h"
#include "motor_control.h"
#include "../peripherals/adc.h"
#include "../peripherals/uart.h"
#include "../peripherals/eeprom.h"


struct data_s {
	struct {
		char  **packet;
	} uart;
	
	uint16_t *adc_load;
	uint16_t *adc_volt;
	uint16_t *adc_temp;
};

volatile struct data_s data = {{NULL},NULL,NULL,NULL};
volatile position_s *pos = NULL;
volatile ee_ctrl_s *eectrl = NULL;
volatile sfts_s *limitsP = NULL;

const char ACK[]  PROGMEM = "_ack.";
const char A_OK[]  PROGMEM = "ok!";

#define UART_ACK    uart_putstr_f(ACK);uart_putstr_f(A_OK)

void uart_dispatch(void)
{
	struct data_s *p = &data;
	
	extern errorstate_t err;
	extern mode_t mode;
	
	for (int i = 0; i <= 4; i++)
		{
			if(p->uart.packet[i] != NULL)
			{
				switch (p->uart.packet[i][0])
				{	// je nutn vyvarovat se hodnotm 33 a 95
										
					case 100:
						// odpovd potae na TRC
						mode =  USER;
						err = ITWORKS;
						UART_ACK;
					break;
					
					case 101:
					{
						extern boolean_t _reset;
						_reset = FALSE;
						UART_ACK;
					}						
					break;
					
					case 102:
						setrefpoint();					
						mode = REFERENCE;
					break;
					
					case 103:
					mode = PROGRAM;
					break;
					
					case 104:
					mode = USER;
					break;
					
					case 110:
						uart_putchar('_');
						uart_putchar(110);
						uart_putchar(mode);
						uart_putchar('!');
					break;
					
					case 111:
						uart_putchar('_');
						uart_putchar(111);
						uart_putchar((uint8_t) data.adc_temp[1]);
						uart_putchar((uint8_t) data.adc_temp[1] >> 8);
						uart_putchar('!');	
					break;
					
					// zjisti mezn hodnoty
					case 51:
						uart_putchar('_');
						uart_putchar(51);
						uart_putchar(limitsP->min_volt);
						uart_putchar('!');
					break;
					
					case 52:
						uart_putchar('_');
						uart_putchar(52);
						uart_putchar(limitsP->max_volt);
						uart_putchar('!');
					break;
					
					case 53:
						uart_putchar('_');
						uart_putchar(53);
						uart_putchar(limitsP->max_temp);
						uart_putchar('!');
					break;
					
					case 54:
						uart_putchar('_');
						uart_putchar(54);
						uart_putchar(limitsP->max_load);
						uart_putchar('!');
					break;
					
					// poloha
					case 60:
						uart_putchar('_');
						uart_putchar(60);
					
						for (int i = 0; i < 6; i++)
						{
							uart_putchar((uint8_t) pos->actual[i]);
							uart_putchar((uint8_t) pos->actual[i] >> 8);
						}						
						uart_putchar('!');
					break;
					
					case 61:
						uart_putchar('_');
						uart_putchar(61);
						uart_putchar((uint8_t) pos->actual[0]);
						uart_putchar((uint8_t) pos->actual[0] >> 8);
						uart_putchar('!');
					break;
					
					case 62:
						uart_putchar('_');
						uart_putchar(62);
						uart_putchar((uint8_t) pos->actual[1]);
						uart_putchar((uint8_t) pos->actual[1] >> 8);
						uart_putchar('!');
					break;
					
					case 63:
						uart_putchar('_');
						uart_putchar(63);
						uart_putchar((uint8_t) pos->actual[2]);
						uart_putchar((uint8_t) pos->actual[2] >> 8);
						uart_putchar('!');
					break;
					
					case 64:
						uart_putchar('_');
						uart_putchar(64);
						uart_putchar((uint8_t) pos->actual[3]);
						uart_putchar((uint8_t) pos->actual[3] >> 8);
						uart_putchar('!');
					break;
					
					case 65:
						uart_putchar('_');
						uart_putchar(65);
						uart_putchar((uint8_t) pos->actual[4]);
						uart_putchar((uint8_t) pos->actual[4] >> 8);
						uart_putchar('!');
					break;
					
					case 66:
						uart_putchar('_');
						uart_putchar(66);
						uart_putchar((uint8_t) pos->actual[5]);
						uart_putchar((uint8_t) pos->actual[5] >> 8);
						uart_putchar('!');
					break;
					
					case 70:
						
						for (int j = 0; j < 7; j++)
						{
							pos->stored.ee_cord[j] = p->uart.packet[i][j+1];
						}		
						UART_ACK;
					break;
							
					// nastaven min.volt
					case 150:
						limitsP->min_volt = data.uart.packet[i][1];
						set_ee_limits_data(limitsP);
						UART_ACK;
					break;
					
					// nastaven max.volt
					case 151:
						limitsP->max_volt = data.uart.packet[i][1];
						set_ee_limits_data(limitsP);
						UART_ACK;
					break;
					
					// nastaven max.zaten
					case 152:
						limitsP->max_load = data.uart.packet[i][1];
						set_ee_limits_data(limitsP);
						UART_ACK;
					break;
					
					// nastaven max.teploty
					case 153:
						limitsP->max_temp = data.uart.packet[i][1];
						set_ee_limits_data(limitsP);
						UART_ACK;
					break;
					
					
					// nastavovn jednotlivch motor
					case 200:
						break;
					
					case 201:	
						pos->desired[0] = data.uart.packet[i][1];
						otoc_s_motory();
						UART_ACK;
					break;
					
					case 202:	
						pos->desired[1] = data.uart.packet[i][1];
						otoc_s_motory();
						UART_ACK;
					break;
					
					case 203:	
						pos->desired[2] = (int16_t) data.uart.packet[i][1] | (int16_t)data.uart.packet[i][2] << 8;
						otoc_s_motory();
						UART_ACK;
					break;
					
					case 204:	
						 pos->desired[3] = (int16_t) data.uart.packet[i][1] | (int16_t)data.uart.packet[i][2] << 8;
						otoc_s_motory();
						UART_ACK;
					break;	
					
					case 205:	
						 pos->desired[4] = (int16_t) data.uart.packet[i][1] | (int16_t)data.uart.packet[i][2] << 8;
						otoc_s_motory();
						UART_ACK;
					break; 
					
					case 206:	
						 pos->desired[5] = (int16_t) data.uart.packet[i][1] | (int16_t)data.uart.packet[i][2] << 8;
						otoc_s_motory();
						UART_ACK;
					break; 
					
					// ulo aktualni pozici
					case 210:
						// prvn bajt udv poad souadnice
						eectrl->prg.ee_cordidx = data.uart.packet[i][1];
						// pelijeme souadnice do doasn promn ven ze zsobnku abychom v nm mohli uvolnit pamt
						eectrl->prg.ee_tmpcord.coord.A = (uint16_t) pos->actual[0];
						eectrl->prg.ee_tmpcord.coord.B = (uint16_t) pos->actual[1];
						eectrl->prg.ee_tmpcord.coord.C = (uint16_t) pos->actual[2];
						eectrl->prg.ee_tmpcord.coord.D = (uint16_t) pos->actual[3];
						eectrl->prg.ee_tmpcord.coord.E = (uint16_t) pos->actual[4];
						eectrl->prg.ee_tmpcord.coord.F = (uint16_t) pos->actual[5];
						// zadme o zpis do eeprom
						eectrl->ee_jobticket = 210;
					break;
								
					case 211:	
						// prvn bajt udv poad souadnice kter  se m vyst
						eectrl->prg.ee_cordidx = data.uart.packet[i][1];
						// zadme ten z eeprom
						eectrl->ee_jobticket = 211;
					break;
						
						
					case 220:
						
					break;			
												
					
					// tato subrutina zpracuje poten inicializaci zpisu souadnic do eeprom
					case 10:
						// prvnch 10 bajt je nzev programu
						for (int j = 0; j<10; j++)
						{
							eectrl->prg.ee_tmpprgname[j] = p->uart.packet[i][j+1];	
						}
						// posledn bajt udv poet souadnic
						// todo: z tohoto potu v promn tmpcntofcord se bude odetat kazda prichozi sourd.
						eectrl->prg.ee_tmpcntofcord = p->uart.packet[i][11];	
						
						// zadme o zpis do eeprom
						eectrl->ee_jobticket = 10;
					break;
					
					//	tato subrutina zpracuje pchoz souadnici a ulo ji do eeprom
					case 11:	
						// prvn bajt udv poad souadnice
						eectrl->prg.ee_cordidx = data.uart.packet[i][1];
						// pelijeme souadnice do doasn promn ven ze zsobnku abychom v nm mohli uvolnit pamt
						for (int j = 0; j < 7; j++)
						{
							eectrl->prg.ee_tmpcord.ee_cord[j] = p->uart.packet[i][j+2];
						}						
						// zadme o zpis do eeprom
						eectrl->ee_jobticket = 11;
					break;
					
					//	tato subrutina si vyd informace o programu a potu souadnic
					case 20:
						eectrl->ee_jobticket = 20;
					break;
					
					//	tato subrutina si vyd souadnice konkrtn pozice
					case 21:	
						// prvn bajt udv poad souadnice kter  se m vyst
						eectrl->prg.ee_cordidx = data.uart.packet[i][1];
						// zadme ten z eeprom
						eectrl->ee_jobticket = 21;
					break;
					
					
					
				}
				// uvolnme adresu z ukazatele a udlme msto pro nov paket
				p->uart.packet[i] = NULL;
				
				//#ifdef DEBUG
				//uart_putstr(p->uart.packet[i]);
				//uart_putstr("\r\n");
				//#endif
			}
		}
}

void print_volt(void)
{
	uart_putstr_f(PSTR("VOLT: "));
	uart_put_uint16( (float)*data.adc_volt * 0.037); uart_putchar('V');
	uart_putstr_f(NEWLINE);
}


void print_temp(void)
{
	uart_putstr_f(PSTR("TEMP: "));
	//uart_put_uint8(((temper[1] - 83) >> 2)-20); uart_putstr("C | ");
	uart_put_uint8(((data.adc_temp[1] - 83) >> 2)-20); uart_putstr("C | ");
	uart_put_uint8(((data.adc_temp[2] - 83) >> 2)-20); uart_putstr("C"); uart_putstr_f(NEWLINE);
}


void DATA_init(void)
{
	UART_init(&data.uart.packet);
	data.adc_volt = getADC_voltPtr();
	data.adc_temp = getADC_tempPtr();
	data.adc_load = getADC_loadPtr();
	
	pos = getMCTR_positionStrPtr();
	eectrl = getEEPR_eectrlStrPtr();
	limitsP = getSFTS_limitsPtr();
	
}
