#include <avr/io.h>
#include <avr/interrupt.h>
#include "../config.h"
#include "data_transfer.h"
#include "motor_control.h"
#include "../peripherals/adc.h"
#include "../peripherals/uart.h"
#include "../peripherals/eeprom.h"


struct motor_s {

	uint8_t  load[6];			// hodnota zaten jednotlivch motor
	uint8_t   rpm[6];			// hodnota (ot/min)/30 jednotlivch motor
	uint8_t _0rpm;				// flag nulovch otek
	uint8_t _running;			// stnovac registr pro ovldn bhu motor todo: zamyslet se na registry GPIO
	uint8_t _direction;			// stnovac registr pro ovldn smru oten (0-doprava, 1-doleva)
	uint8_t _dir_change;		// flag zmny smru
};	


extern mode_t mode;
struct motor_s motor = {{0},{0},0,0,0,0};
volatile position_s position = {{0},{0},0,0,{0}};


volatile position_s  *getMCTR_positionStrPtr(void){ return &position ;}


void MCTR_init (void)
{
	// DIR
	DDRA = 0b11111111;	PORTA = 0b00000000;
	// EN
	DDRC = 0b11111111;	PORTC = 0b00000000;
	
	// nastavme vechny piny hallovek do vstupnho stavu
	DDRE = 0b0000010;
	// inicializace peruen registru PB0
	EIMSK  |= (1 << PCIE0);		// povol peruen od registru PB0 piny (PB0-PB5)
	PCMSK0 |= (1<<PCINT2) |		// DC1_HALL
			  (1<<PCINT3) |		// DC2_HALL
			  (1<<PCINT4) |		// DC3_HALL
			  (1<<PCINT5) |		// DC4_HALL
			  (1<<PCINT6) |		// DC5_HALL
			  (1<<PCINT7);		// DC6_HALL
}


//                  _                               _             _ 
//  _ __ ___   ___ | |_ ___  _ __    ___ ___  _ __ | |_ _ __ ___ | |
// | '_ ` _ \ / _ \| __/ _ \| '__|  / __/ _ \| '_ \| __| '__/ _ \| |
// | | | | | | (_) | || (_) | |    | (_| (_) | | | | |_| | | (_) | |
// |_| |_| |_|\___/ \__\___/|_|     \___\___/|_| |_|\__|_|  \___/|_|
//                                                                  

// zde je poteba se zamyslet jesli neme vzniknout njak nednouc stav tm e jsou p running a direction nekonzistentn
void stinovani (void)
{	static uint8_t _p_running = 0;
	static uint8_t _p_direction = 0;
	
	// pokud dolo nkde ke zmn
	if ((_p_direction != motor._direction) || (_p_running != motor._running))
	{ 
		
		/*	petvome logiku pro L298ku podle tto pravdivostn tabulky
		 _______________
		|_D_|_E_|_A_|_B_|
		| 0 | 0 | 0 | 0 |
		| 0 | 1 | 1 | 1 |
		| 1 | 0 | 1 | 0 |
		|_1_|_1_|_0_|_1_|  */
		
		pinswap_u remap_act = {{motor._running ^ motor._direction, motor._direction}};
		pinswap_u remap_new = {{0,0}};
		
		// pimapovn dcch vstup na pslun piny procesoru
		remap_new.bit.b0 = remap_act.bit.b4;
		remap_new.bit.b1 = remap_act.bit.b12;
		remap_new.bit.b2 = remap_act.bit.b5;
		remap_new.bit.b3 = remap_act.bit.b13;
		remap_new.bit.b4 = remap_act.bit.b2;
		remap_new.bit.b5 = remap_act.bit.b10;
		remap_new.bit.b6 = remap_act.bit.b3;
		remap_new.bit.b7 = remap_act.bit.b11;
		remap_new.bit.b12 = remap_act.bit.b9;
		remap_new.bit.b13 = remap_act.bit.b1;
		remap_new.bit.b14 = remap_act.bit.b8;
		remap_new.bit.b15 = remap_act.bit.b0;
		
		PORTC = remap_new.pinmap[0];
		PORTA = remap_new.pinmap[1];
		
		_p_direction = motor._direction;
		_p_running = motor._running;
	}
}

void setrefpoint (void)
{
	EIMSK  &= ~ (1 << PCIE0);
	motor._direction = 0b11111111;
	motor._running = 0b11111111;
}

void positionNull (void)
{	 
	motor._direction = 0b00000000;
	motor._running = 0b00000000;
	
	position.actual[0] = 0;
	position.actual[1] = 0;
	position.actual[2] = 0;
	position.actual[3] = 0;
	position.actual[4] = 0;
	position.actual[5] = 0;
}

// void print_load (void)
// {	 
// 	// vytiskneme si aktuln postaven motor	
// 	uart_putstr_f(PSTR("LOAD: "));
// 	for (int i = 0; i < 5; i++)
// 	{ uart_put_uint16(motor.p_load[i]); uart_putchar('|'); }
// 	uart_put_uint16(motor.p_load[5]); uart_putstr_f(NEWLINE);
// }

void print_position (void)
{	 
	// vytiskneme si aktuln postaven motor	
	uart_putstr_f(PSTR("POS: "));
	for (int i = 0; i <= 4; i++)
	{ uart_put_uint16(position.actual[i]); uart_putchar('|'); }
	uart_put_uint16(position.actual[5]); uart_putstr_f(NEWLINE);
}

void calc_RPM (void)
{	
	static uint16_t prev_position[6] = {0};
	
	for(int i = 0; i < 6; i++)
	{	// zjistme poet pulz za uritou asovou konstantu (*30 = hodnota RPM)
		motor.rpm[i] = (prev_position[i] - position.actual[i]);
		// ulome si momentln hodnotu  
		prev_position[i] = position.actual[i];
		// zjistme zda se motor ji neto a pokud ne tak nastavme nulu
		if (motor.rpm[i] == 0)
		  { motor._0rpm &= ~(1<<i); }	// pokud jsou zmny nulov zapi to
		else 
		  { motor._0rpm |=  (1<<i); }	// pro jistotu i naopak
	}
}

void print_rpm (void)
{	 
	// vytiskneme si aktuln postaven motor	
	// vynsoben 32 >>4 pomoc bitovho posunu (sprvn jen 30, chyba +- 60RPM)
	uart_putstr_f(PSTR("RPM: "));
	for (int i = 0; i < 6; i++)
	{
		uart_put_uint16(motor.rpm[i]); uart_putchar('|'); 
	}
	uart_putstr("!");
}


void load_nextcord (uint8_t x)
{	
	switch (x)
	{
		case 1:
		{   // nateme si souadnici
			static uint8_t last_read = 0;
			coord_s cord_tmp = {0};
	
			eeprm_read_cord(&cord_tmp, last_read);
	
			position.desired[0] = cord_tmp.A;
			position.desired[1] = cord_tmp.B;
			position.desired[2] = cord_tmp.C;
			position.desired[3] = cord_tmp.D;
			position.desired[4] = cord_tmp.E;
			position.desired[5] = cord_tmp.F;	
			break;	
		}
		
		case 2:
		break;
	}
	
}


const uint16_t program[5][6] ={
{20, 20, 20,  20, 20,  20},
{70, 110, 350, 200, 200, 200},
{20, 70, 250, 120, 120, 500},
{70, 20, 20, 20, 50, 20},
{20, 60, 110, 100, 90, 300},
};

void next_cord (void)
{
	static int next = 0;
	
	position.desired[0] = program[next][0];
	position.desired[1] = program[next][1];
	position.desired[2] = program[next][2];
	position.desired[3] = program[next][3];
	position.desired[4] = program[next][4];
	position.desired[5] = program[next][5];
	next++;
	
	if (next >= 5) next = 0;
}

void getpos (void)
{
	position.desired[0] = position.stored.coord.A;
	position.desired[1] = position.stored.coord.B;
	position.desired[2] = position.stored.coord.C;
	position.desired[3] = position.stored.coord.D;
	position.desired[4] = position.stored.coord.E;
	position.desired[5] = position.stored.coord.F;
}


int otoc_s_motory (void)
{	
	// inicializujeme si pouit promnn
	uint8_t i = 0;				
	uint8_t _dir_tmp = 0b00000000;
	uint8_t _en_tmp = 0b00000000;
	
	// zkontrolujeme zda bylo dosaeno poadovan souadnice
	if (position._reached) { return 0;  }

//	next_cord();
	_delay_ms(200);
	if (mode == PROGRAM) next_cord(); 	
	// projdem jednotliv motory a vyhodnotme smysl jejich budoucho oten
	for(i = 0; i < 6; i++)
	{	// pokud je poadovan hel natoen vt ne aktuln
		if ( position.desired[i] > position.actual[i]) 
		{	// nastavme smr oten
			_dir_tmp &= ~ (1<<i);
			// do motoru pustme tvu
			_en_tmp  |= (1<<i);
			// nastavme si maskovac promnou
			position._reached |= (1<<i);
		}	 // pokud je poadovan hel men ne aktuln
		else if ( position.desired[i] < position.actual[i] ) 
		{	// nastavme smr oten
			_dir_tmp |= (1<<i);
			// do motoru pustme tvu
			_en_tmp  |= (1<<i);
			position._reached |= (1<<i);
		} 
		else 
		{	// v ppad e nen ani jedna podmnka splnna motor je v poadovan pozici
			position._reached &= ~(1<<i);
		}
	}
	
	// podvme se na kterch motorech dolo ke zmn smru
	motor._dir_change = motor._direction ^ _dir_tmp;
	// pepeme doasn promn do ivch
	motor._direction = _dir_tmp;
	motor._running = _en_tmp;
	motor._0rpm = _en_tmp;
	// vydn nov souadnice
	//if (mode == PROGRAM) uart_putstr("_F!");
	// vratme si true, funkce probhla a sem
	return 1;
}



// funkce vracejc abolutn hodnotu 15bit sla
// udlat inline, mon... poteba promyslet na kolika mistech bude pouito
inline int abs(int val)
{
    return (val + (val >> 15)) ^ (val >> 15);
}


// hodnota se na reln RPM bude pepotvat jen pi tisku	
// sem se bude ukldat jen hodnota zmny za urit asov okamik
// + se bude plnit promn nulovho RPM pro poteby dcho algorytmu



//                  _ _   _                                                   _             
//  _ __   ___  ___(_) |_(_) ___  _ __    _ __ ___   ___  __ _ ___ _   _ _ __(_)_ __   __ _ 
// | '_ \ / _ \/ __| | __| |/ _ \| '_ \  | '_ ` _ \ / _ \/ _` / __| | | | '__| | '_ \ / _` |
// | |_) | (_) \__ \ | |_| | (_) | | | | | | | | | |  __/ (_| \__ \ |_| | |  | | | | | (_| |
// | .__/ \___/|___/_|\__|_|\___/|_| |_| |_| |_| |_|\___|\__,_|___/\__,_|_|  |_|_| |_|\__, |
// |_|                                                                                |___/ 
// 
	
ISR(SIG_PIN_CHANGE0)
{  
	static uint8_t act_portprint, old_portprint;
		   uint8_t i, changes;        
	
	// ulome si otisk portu
	act_portprint  = PINE >> 2;
	// vymaskujeme si zmny a ulome										
	changes = act_portprint ^ old_portprint;
	
	for(i=0; i<6; i++) 
	{ 
		// pokud dolo na pinu ke zmn logick urovn
		if (changes & (1<<i))			
		{	// a jedn-li se o sestupnou hranu !! zmna z dvodu invertoru ve midtov ko
			if (!(act_portprint & (1<<i)))
			{	// pedej tuto skutenost k nsledn kontrole
				position._validity |= (1<<i);
				motor._dir_change &= ~ (1<<i); 
			}
			// jedn-li se o nabeznou hranu
			else
			{	// zkontroluj validitu a zmnu smru
				// jestlie se od poslednho otoen zmnil smr, pulz je nutn ignorovat
				// nen dokonen ve shodnm smru, tud je neplatn
				if ((position._validity & (1<<i))) 
				{
					if(!(motor._dir_change & (1<<i)))
					{	// pokud je smr CW (po smru hodinovych ruiek)
						if (motor._direction & (1<<i))
						{	// piti tento pulz
							position.actual[i]--;
							// zhodnotme pozice a ppadn zastavme motor 
							if (position.actual[i] <= position.desired[i])
							{	// motor zastavme pes stnovac "registr" 
								motor._running &= ~ (1<<i);
								// a nastavme flag o dozaen pozici
								position._reached &= ~(1<<i);
							//	uart_put_uint16(position.desired[i]);
							}
						}
						// pokud je smr CCW (proti smru hodinovych ruiek)
		    			else
						{	// odeti tento pulz
							position.actual[i]++;
							// zhodnotme pozice a ppadn zastavme motor 
							if (position.actual[i] >= position.desired[i])
							{	// motor_stop(i); todo:dodlat stnovn na druh stran
								motor._running &= ~ (1<<i); 
								// pozice nastavena, naho flag
								position._reached &= ~(1<<i);
								//uart_put_uint16(position.desired[i]);
							}				
						}
						// zhodme validan bit
						position._validity &= ~ (1<<i);
					}
					else
					{   // nkter z podmnek selhala, nbn hrana nbyla zaznamenna
						// nebo dolo ke zmn smru
						position._validity &= ~ (1<<i);
						// a protoe zmna nyn probhla, je nutn smazat jej flag
						motor._dir_change &= ~ (1<<i); 
					}	
				}
				
			}				
		}
	}
	// aktuln otisk portu ulome
	old_portprint = act_portprint; 
}