
#include "safety_system.h"
#include "../config.h"
#include "../peripherals/eeprom.h"
#include "../peripherals/adc.h"

#define FAN_PWM	OCR0A
#define MAX	255
#define MIN	0


volatile sfts_s limits = {5,15,50,150}; 

volatile uint16_t *adc_volt = NULL;
volatile uint16_t *adc_load = NULL;
volatile uint16_t *p_temp = NULL;



extern errorstate_t err;
extern mode_t mode;


volatile sfts_s  *getSFTS_limitsPtr(void){ return &limits; }

void fanSet (uint8_t fanspeed)
{

	if(fanspeed == 0 || fanspeed == 255)
	{
		TCCR0A &= ~ (1<<COM0A1);
		FAN_PORT = (FAN_PORT & ~(1<<FAN_PIN)) | ((!!fanspeed)<<FAN_PIN);
	}
	else
	{
		TCCR0A |= (1<<COM0A1);
		FAN_PWM = fanspeed;
	}
}


void SFT_init (void)
{	// naten meznch hodnot z EEPROM
	//get_ee_limits_data(&limits);
	adc_volt = getADC_voltPtr();
	adc_load = getADC_loadPtr();
	p_temp = getADC_tempPtr();
}

void safety_check (void)
{	
	static uint8_t i = 0;
	switch (i++)
	{
		case 0:
			// kontrola min max napt
			{
				uint8_t res = 0;
				// pepotat opan
				res = ((float)*adc_volt * 0.037);
				if (!(res > limits.min_volt && res < limits.max_volt)) {err = ADC_ERR; mode = ERROR;}
			}			
		break;
	
		case 1:
			{
				// kontrola teploty
				uint8_t res = 0;
				res =  (((p_temp[1] - 83) >> 2)-20);
				if (res > limits.max_temp){ err = TEMP_ERR; fanSet(MAX); mode = ERROR;}
				res = (((p_temp[2] - 83) >> 2)-20);
				if (res > limits.max_temp){ err = TEMP_ERR; fanSet(MAX); mode = ERROR;}
			}			
		break;
	
		case 2:
			// kontrola zaten
			for(int i=0; i < 6; i++)
			{
				if(adc_load[i] > limits.max_load){ err = LOAD_ERR; fanSet(MAX); mode = ERROR;}
			}
		break;
		
		default:
		i=0;
		break;
	}
			
}	
 


