#include "../config.h"
#include "safety_system.h"
#include "data_transfer.h"
#include "motor_control.h"
#include "../peripherals/adc.h"
#include "../peripherals/uart.h"
#include "../peripherals/timers.h"
#include "../peripherals/eeprom.h"

#define GPIO_t 10		// stnovn port co 10ms
#define DATA_t 10		// kontrola datov schrnky co 10ms 
#define ADC_t  10		// slzvn hodnot z ADC co 10ms
#define USER_t 200 		
#define PRG_t 200 		
#define WAIT_t 250
#define SFTS_t 250
#define REFERENCE_t 10000

#define RXTX_LED_delay 10	//  hodnota zpodn * rychlost stnovn port (10ms)

struct timer_s
{
	uint8_t adc;
	uint8_t gpio;
	uint8_t data;
	uint8_t program;
	uint8_t user;		
	uint8_t wait;
	uint8_t safety;
	uint16_t reference;
};

extern errorstate_t err;
extern mode_t mode;

struct timer_s timer = {0,0,0,0,0,0};
volatile boolean_t *adc_idle = NULL;

volatile boolean_t *rxLEDptr = NULL;
volatile boolean_t *txLEDptr = NULL;

void GPIO_init (void)
{
	rxLEDptr = getUART_rxLEDptr(); 
	txLEDptr = getUART_txLEDptr();	
	
	ERR_LED_SETUP;
	FAN_SETUP;
	fanSet(100);
}

void TASK_init (void)
{   
	ADC_init(&adc_idle); 
	DATA_init();
	EEPRM_init();
	MCTR_init();
	TIME_init();
	GPIO_init();
	SFT_init();
	//uart_putstr_f(FIRM_INFO);
	mode = PC_WAIT;
}



void task_dispatch (void)
{	
	// proje vechny asovae a vyi koly
	switch(mode)
	{
		case ERROR:
			timer.gpio++;
			timer.data++;
		break;
		
		case PC_WAIT:
			err = PC_ACK;
			timer.wait++;
			timer.data++;
			timer.gpio++;
		break;
		
		case USER:
			timer.adc++;
			timer.gpio++;
			timer.data++;
			timer.safety++;
			//timer.user++;
		break;
		
		case PROGRAM:
			timer.adc++;
			timer.gpio++;
			timer.data++;
			timer.safety++;
			timer.program++;
		break;
		
		case REFERENCE:
			timer.gpio++;
			timer.safety++;
			timer.reference++;
		break;
	}
		
		if (timer.gpio >= GPIO_t) 
		{	
			timer.gpio = 0;
			stinovani();
			
			
			const uint8_t err_led_delays[CNT_OF_ERR] = {80,70,60,50,40,30,20};
 			
			 if (err)
			 {
				static uint8_t err_led_timer = 0;
 				if (err_led_timer++ >= err_led_delays[err-1])
				{ 			
					ERR_LED_NEG;				
 					err_led_timer = 0;
 				}
			 }
			 else
			 {
				ERR_LED_OFF;				 
			 }
			
			
			static uint8_t uart_led_timer = 0;
			if (uart_led_timer++ >= RXTX_LED_delay)
			{
				// indikace pijmu a vysln
				if (!*rxLEDptr) {RX_LED_OFF; *rxLEDptr = FALSE;}
				if (!*txLEDptr) {TX_LED_OFF; *txLEDptr = FALSE;}
				uart_led_timer = 0;
			}
			
		}
	
		if (timer.data >= DATA_t) 
		{
			timer.data = 0;
			uart_dispatch();
			eeprm_dispatch();
		}
		
		if (timer.reference >= REFERENCE_t) 
		{
			timer.reference = 0;
			positionNull();
			char datagram[3] = {'_',102,'!'};
			uart_putstr(datagram);
			EIMSK  |= (1 << PCIE0);
			mode = USER;
			
			
		}
		
		if (timer.safety >= SFTS_t) 
		{	
			timer.safety = 0;
			safety_check();
		}
		
		if (timer.adc >= ADC_t) 
		{	
			if (*adc_idle)
			{ ADC_dispatcher();	}
			else { timer.adc = 5; } // zkrcen ekn v ppad zaneprzdnnho adc
		}
							
		
		if (timer.wait >= WAIT_t) 
		{	// vynuluj asova	
			timer.wait = 0;
			uart_putstr_f(PSTR("_d!"));
		}
		
// 		if (timer.user >= USER_t) 
// 		{	// vynuluj asova	
// 			timer.user = 0;
// 		uart_putstr_f(PSTR("HI"));
// 		}	
		
		if (timer.program >= PRG_t) 
		{	// vynuluj asova	
			timer.program = 0;
			otoc_s_motory() ;
		}	

}

