#include <avr/io.h>
#include <avr/interrupt.h>
#include "../config.h"
#include "uart.h"


struct adc_s {
	
	boolean_t idle;
	uint8_t   dmx_idx:4;
	uint8_t   channel_idx:4;
	
	struct {
		uint16_t  load[6];
		uint16_t  temp[3];
		uint16_t  volt;
	} rawval;
};

volatile struct adc_s adc = {TRUE,0,0,{{0},{0},0}};

//volatile boolean_t *getADC_idlePtr(void){ return &adc.idle; }
volatile uint16_t  *getADC_loadPtr(void){ return adc.rawval.load; }
volatile uint16_t  *getADC_tempPtr(void){ return adc.rawval.temp; }
volatile uint16_t  *getADC_voltPtr(void){ return &adc.rawval.volt; }

void ADC_init (volatile boolean_t **p_idle)
{
	// nastavme vechny piny ADC do vstupnho stavu
	ADC_DDR  &= ~((1 << DC1_MEASURE) |
				  (1 << DC2_MEASURE) |
				  (1 << DC3_MEASURE) |
				  (1 << DC4_MEASURE) |
				  (1 << DC5_MEASURE) |
				  (1 << DC6_MEASURE) |
				  (1 << PWR_MEASURE) |
				  (1 << TEMP_MEASURE));
	ADC_PORT &= ~((1 << DC1_MEASURE) |
				  (1 << DC2_MEASURE) |
				  (1 << DC3_MEASURE) |
				  (1 << DC4_MEASURE) |
				  (1 << DC5_MEASURE) |
				  (1 << DC6_MEASURE) |
				  (1 << PWR_MEASURE) |
				  (1 << TEMP_MEASURE));
				  
	// nastavme DMX
	DMX_DDR  |= ((1 << DMX_A) |(1 << DMX_B) | (1 << DMX_C));
	DMX_PORT &= ~((1 << DMX_A) |(1 << DMX_B) | (1 << DMX_C));
	
	// povolme ADC, nastavme 128 dliku (125KHz pi 16MHz krystalu = optimal) a povolme peruen od ADC
	ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0) | (1<<ADIE);
	// zakeme cel port jako logick
	DIDR0 = 0b11111111;
	// pedme odkaz na pole hodnot
	//volatile struct adc_s *p = &adc;
	*p_idle = &adc.idle;
}



// pokud zbyde as zkusit tohle
// void ReadADC_Request(uint8_t _ADchannel, boolean_t repetedly = SINGLE_READ)
//SINGLE_READ
//LOOP_READ

void ReadADC_Request(uint8_t _ADchannel) 
{
	if (bit_is_set(ADCSRA,ADSC))	
	{
		// poteba oznamit e pevod nen registrovn protoe se zrovna pracuje na pevodu
		// asi error melo by to byt tak ze se sem to za normalnich okolnosti nemuze dostat
	}
	else
	{	// nastavme ADC
		adc.channel_idx = _ADchannel;			   // index tenho kanlu
		ADMUX = (ADMUX & 0x11110000) | _ADchannel; // MUX kanlu
		ADCSRA |= _BV(ADSC);                       // spustme pevod
		adc.idle = FALSE;
	}
}

void ADC_dispatcher (void)
{  	// dle udlat kontrolu na rozsah	
	if (adc.channel_idx == 7)
	{  // nastavme multiplexer
		if (adc.dmx_idx > 2) adc.dmx_idx = 0;
		DMX_PORT = (DMX_PIN & 0x11111000) | (adc.dmx_idx << 5);
		// spustme pevod
		ReadADC_Request(adc.channel_idx);
	}				
	else
	{   ReadADC_Request(adc.channel_idx); }
}

ISR(ADC_vect,ISR_NOBLOCK)
{
	if (adc.channel_idx <= 5 )
	 {
		 // korekce chyby kter se ze zatm neznmch pin objevila... asi patn nvrh ploku
		 if (adc.channel_idx >= 4) {adc.rawval.load[adc.channel_idx] = ADC-100;}
		else {adc.rawval.load[adc.channel_idx] = ADC;}
		adc.channel_idx++;
	 }
	else if (adc.channel_idx == 6)
	 {
		adc.rawval.volt = ADC;
		adc.channel_idx++;
	 }
	else if (adc.channel_idx == 7)
	 {
		adc.rawval.temp[adc.dmx_idx] = ADC;
		adc.dmx_idx++;
		adc.channel_idx = 0;
	 }
	else
	 {  // kdyby nastala njak chyba
		adc.channel_idx = 0;
	 }  
	
	adc.idle = TRUE;
}



