#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <string.h>
#include "../config.h"
#include "uart.h"
#include "eeprom.h"
#include "../controls/safety_system.h"

// povolen peruen od ready eeprom
#define EEREADY_ENA EECR |= (1<<EERIE);
#define EEREADY_DIS EECR &= ~(1<<EERIE);

// ATmega168 - 512byte eeprom pamti
// 3 bytes signatura
// 10 bytes nazev programu
// 70 souadnic seere 490 bytes
// ---- vyuito celkem 511 bytes z 512bytes

struct ee_storage_s {
	char signature[SIZEOF_SIGN];
	sfts_s limits_data;
	char prog_name[SIZEOF_PROG_NAME];
	uint8_t numofcoords;
	coord_s coord[COUNT_COORDS];
};

// EEPROM data
struct ee_storage_s EEMEM ee_storage = {{0},{0},{0},0,{{0}}};
ee_ctrl_s eeprm = {0,FALSE,FALSE,{FALSE,{0},0,0,{{0}}}};


ee_ctrl_s *getEEPR_eectrlStrPtr(void){ return &eeprm; }

// EEPROM functions
ISR(EE_READY_vect)
{
	eeprm.ee_idle = TRUE;
	EEREADY_DIS;
}

void EEPRM_init (void)
{
	char *signature = "TRC";
	char ee_signature_temp[SIZEOF_SIGN] ;
	eeprm.ee_idle = FALSE;
	// pokme na eeprom pamt
	if ( !eeprom_is_ready() ) { eeprom_busy_wait(); }
	// pokusme se pest signaturu, otagovn eeprom pamti
	eeprom_read_block(&ee_signature_temp, &ee_storage.signature, SIZEOF_SIGN);	
	// jestlie signatura uloen v eeprom sed s uloenou ve flash
	if ( memcmp (ee_signature_temp, signature, SIZEOF_SIGN) == 0 )		
	{	// super mme ji tam, meme zkontrolovat konzistenci etc ?
			
	}
	else // eeprom jet zformtovnu nemme, ppadn je nakopnut 
	{	// pokud mme zapnut debug, oznmme uivateli formtovn EEPROM pamti
		#ifdef DEBUG
			uart_putstr_f(EEP_FORMAT);
		#endif
		// eeprom si otagujeme
		eeprom_write_block(signature, &ee_storage.signature, SIZEOF_SIGN);	
		// a zformtujeme
		char tmp_prg_name[SIZEOF_PROG_NAME] = "BLANK";
		// zapeme "przdno" do nzvu programu
		eeprom_write_block(&tmp_prg_name, &ee_storage.prog_name, SIZEOF_PROG_NAME);
		// zapeme nulov poet souadnic
		uint8_t tmp_numofcords = 0;
		eeprom_write_block(&tmp_numofcords, &ee_storage.numofcoords, 1);
		// pipravme si nulovou souadnici
		coord_s null_cords = {0,0,0,0,0,0};
		// tedy proeneme eepromkou nulov hodnoty
		for (int i = 0; i < COUNT_COORDS; i++)
		{	
			eeprom_write_block(&null_cords, &ee_storage.coord[i], SIZEOF_COORDS );
		}
	}
	EEREADY_ENA;
}

void get_ee_limits_data(volatile sfts_s *sPtr )
{
	eeprom_read_block(sPtr, &ee_storage.limits_data, SIZEOF_SFTS);
}

void set_ee_limits_data(volatile sfts_s *sPtr )
{
	eeprom_update_block(sPtr,&ee_storage.limits_data,SIZEOF_SFTS);
}


void eeprm_write_cord(coord_s *p_coord, uint8_t where)
{
	eeprom_update_block(p_coord,&ee_storage.coord[where],SIZEOF_COORDS);
}

// touto funkc budeme vracet souadnice z poadovanho msta v pamti eeprom 

void eeprm_read_cord(coord_s *p_coord, uint8_t wherefrom)
{
	eeprom_read_block(p_coord,&ee_storage.coord[wherefrom],SIZEOF_COORDS);
}

void eeprm_dispatch(void)
{
	if (eeprm.ee_idle)
	{
		switch (eeprm.ee_jobticket)
		{	
			case 10:
			eeprm.ee_idle = FALSE;
			// zapeme jmno programu
			eeprom_update_block(&eeprm.prg.ee_tmpprgname, &ee_storage.prog_name, SIZEOF_PROG_NAME);
			// zapeme pedpokldan poet souadnic
			eeprom_update_block(&eeprm.prg.ee_tmpcntofcord, &ee_storage.numofcoords, 1);
			// odeleme aplikaci zprvu o spnm zpisu do eeprom
			uart_putstr_f(PSTR("_PRGWR!"));
			
			eeprm.ee_jobticket = 0;
			EEREADY_ENA;
			#ifdef DEBUG
			uart_putstr_f(EEP_PRGNAME);
			uart_putstr_f(EEP_WRITE);
			#endif
			break;
			
			case 11:
			eeprm.ee_idle = FALSE;
			eeprm_write_cord(&eeprm.prg.ee_tmpcord.coord, eeprm.prg.ee_cordidx);
			// odeleme aplikaci zprvu o spnm zpisu do eeprom
			uart_putstr_f(PSTR("_CORDWR!"));
			
			eeprm.ee_jobticket = 0;
			EEREADY_ENA;
			#ifdef DEBUG
			uart_putstr_f(EEP_COORD);
			uart_putstr_f(EEP_WRITE);
			#endif
			//eeprm_read_cord(&eeprm.prg.ee_tmpcord, eeprm.prg.ee_cordidx);
			break;
			
			case 210:
			eeprm.ee_idle = FALSE;
			eeprm_write_cord(&eeprm.prg.ee_tmpcord.coord, eeprm.prg.ee_cordidx);
			// odeleme aplikaci zprvu o spnm zpisu do eeprom
			uart_putchar(95);
			uart_putchar(210);
			uart_putchar(eeprm.prg.ee_cordidx);
			uart_putchar(33);
			
			eeprm.ee_jobticket = 0;
			EEREADY_ENA;
			#ifdef DEBUG
			uart_putstr_f(EEP_COORD);
			uart_putstr_f(EEP_WRITE);
			#endif
			break;
			
			case 211:
			eeprm.ee_idle = FALSE;
			eeprom_read_block(&eeprm.prg.ee_tmpcord,&ee_storage.coord[eeprm.prg.ee_cordidx],SIZEOF_COORDS);
			// sestavme paket a odeleme.. todo: nejakou kontrolu ?
			uart_putchar('_');
			for (int i = 0; i < 7; i++)
			{
				uart_putchar(eeprm.prg.ee_tmpcord.ee_cord[i]);
			}		
			uart_putchar('!');
			
			eeprm.ee_jobticket = 0;
			EEREADY_ENA;
			//#ifdef DEBUG
			//uart_putstr("Coordinates sucesfully written in EEPROM.\r\n");
			//#endif
			//eeprm_read_cord(&p->eeprm.prg.ee_tmpcord, p->eeprm.prg.ee_cordidx);
			break;
			
			
			case 20:
			eeprm.ee_idle = FALSE;
			eeprom_read_block(&eeprm.prg.ee_tmpprgname, &ee_storage.prog_name, SIZEOF_PROG_NAME);
			// zapeme pedpokldan poet souadnic
			eeprom_read_block(&eeprm.prg.ee_tmpcntofcord, &ee_storage.numofcoords, 1);
			// sestavme zprvu a paket odeleme
			uart_putchar('_');
			uart_putstr(eeprm.prg.ee_tmpprgname);
			uart_putchar(eeprm.prg.ee_tmpcntofcord);		
			uart_putchar('!');
			
			eeprm.ee_jobticket = 0;
			EEREADY_ENA;
			//#ifdef DEBUG
			//uart_putstr("PrgName sucesfully written in EEPROM.\r\n");
			//#endif
			break;
			
			case 21:
			eeprm.ee_idle = FALSE;
			eeprom_read_block(&eeprm.prg.ee_tmpcord,&ee_storage.coord[eeprm.prg.ee_cordidx],SIZEOF_COORDS);
			// sestavme paket a odeleme.. todo: nejakou kontrolu ?
			uart_putchar('_');
			for (int i = 0; i < 7; i++)
			{
				uart_putchar(eeprm.prg.ee_tmpcord.ee_cord[i]);
			}		
			uart_putchar('!');
			
			eeprm.ee_jobticket = 0;
			EEREADY_ENA;
			//#ifdef DEBUG
			//uart_putstr("Coordinates sucesfully written in EEPROM.\r\n");
			//#endif
			//eeprm_read_cord(&p->eeprm.prg.ee_tmpcord, p->eeprm.prg.ee_cordidx);
			break;
			
		}
	}
	else
	{
		EEREADY_ENA;
	}
}