#include <inttypes.h>
#include <string.h>
#include <stdbool.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "uart.h"
#include "../config.h"
#include "../controls/data_transfer.h"

#define IS_END_CHAR(x) ((x=='\n')||(x=='\r'))
#define CHAR 0
#define INT  1
#define LONG 2


struct uart_s
{
	struct {
		char    buffer[TXBUFFLEN];
		uint8_t write_idx;
		uint8_t send_idx;
		uint8_t left2send;
		boolean_t dataontheway; 
	} tx;

	struct {
		char       buffer[RXBUFFLEN];
		uint8_t    write_idx;
		uint8_t    pkt_idx;
		char      *packet[NUMOFPACKET];
		boolean_t  dataontheway;
	} rx;
};

volatile struct uart_s uart = {{{0},0,0,0,0},{{0},0,0,{NULL},0}};	

boolean_t rx_led_blik = FALSE;
boolean_t tx_led_blik = FALSE;

boolean_t *getUART_rxLEDptr(void){ return &rx_led_blik; }
boolean_t *getUART_txLEDptr(void){ return &tx_led_blik; }


void uart_starttx(void)
{
    int8_t nextidx = uart.tx.send_idx +1;
    if (nextidx >= TXBUFFLEN) nextidx = 0;
    UDR0 = uart.tx.buffer[uart.tx.send_idx];
    uart.tx.send_idx = nextidx;
	uart.tx.left2send--;
}

void uart_putchar(char c)
{
    while (!((TXBUFFLEN - uart.tx.left2send) >= 1 )) { _delay_us(10);}
	int8_t nextidx = uart.tx.write_idx +1;
    if (nextidx >= TXBUFFLEN) nextidx = 0;
    uart.tx.buffer[uart.tx.write_idx] = c;
    uart.tx.write_idx = nextidx;
	uart.tx.left2send++;
	if (!uart.tx.dataontheway)  { uart.tx.dataontheway = TRUE; uart_starttx(); }
}

void uart_putstr(const char *s)
{
    uint8_t len = strlen(s);
    while (!((TXBUFFLEN - uart.tx.left2send) > len )) { _delay_us(10);}
	
	for (int i = 0; i < len; i++)
    {
        int8_t nextidx = uart.tx.write_idx +1;
		if (nextidx >= TXBUFFLEN) nextidx = 0;
		uart.tx.buffer[uart.tx.write_idx] = s[i];
		uart.tx.write_idx = nextidx;
	}
	uart.tx.left2send += len ;
	if (!uart.tx.dataontheway)  { uart.tx.dataontheway = TRUE; uart_starttx(); }
}

void uart_putstr_f(const char *s)
{
  
  while (pgm_read_byte(&(*s)))
  uart_putchar(pgm_read_byte(&(*s++)));
}

int uart_printf(char c, FILE *stream)
{
    while (!((TXBUFFLEN - uart.tx.left2send) >= 1 )) { _delay_us(10);}
	int8_t nextidx = uart.tx.write_idx +1;
	if (nextidx >= TXBUFFLEN) nextidx = 0;
    uart.tx.buffer[uart.tx.write_idx] = c;
    uart.tx.write_idx = nextidx;
	uart.tx.left2send++;
	if (!uart.tx.dataontheway)  { uart.tx.dataontheway = TRUE; uart_starttx(); }
	return 0;
}

FILE myStdOut = FDEV_SETUP_STREAM( uart_printf, NULL, _FDEV_SETUP_WRITE);


void uart_put_uint8(uint8_t val) 
{
	char buf[3];
	int8_t ptr;
	for(ptr = 0; ptr < 3; ++ptr) {
		buf[ptr] = (val % 10) + '0';
		val /= 10;
	}
	for(ptr = 2; ptr > 0; --ptr) {
		if (buf[ptr] != '0') break;
	}
	for(;ptr>=0;--ptr) {
		uart_putchar(buf[ptr]);
	}
	
}

void uart_put_uint16(uint16_t val) {
	char buf[5];
	int8_t ptr;
	for(ptr = 0; ptr < 5; ++ptr) {
		buf[ptr] = (val % 10) + '0';
		val /= 10;
	}
	for(ptr = 4; ptr > 0; --ptr) {
		if (buf[ptr] != '0') break;
	}
	for(;ptr>=0;--ptr) {
		uart_putchar(buf[ptr]);
	}
	
}
/*
void uart_put_float(float val) {


	const char print [] = "00.0 V";
	print[0] = (val / 100) + '0';
    print[1] = (val / 10) % 10 + '0';
    print[3] = val % 10 + '0';
	uart_putstr(print);

}*/

void uart_put_hex( char dataType, unsigned long data )
{
char count, i, temp;
char dataString[] = "0x        ";

if (dataType == CHAR) count = 2;
if (dataType == INT) count = 4;
if (dataType == LONG) count = 8;

for(i=count; i>0; i--)
{
  temp = data % 16;
  if((temp>=0) && (temp<10)) dataString [i+1] = temp + 0x30;
  else dataString [i+1] = (temp - 10) + 0x41;

  data = data/16;
}

uart_putstr(dataString);
}


/// uart initialization
void UART_init (char ***packet)
{
    // ----- nastavme port ---------
	 RX_DDR &= ~ (1 << RX_PIN);
	 TX_DDR |=   (1 << TX_PIN);
	 RX_LED_DDR |= (1 << RX_LED);
	 TX_LED_DDR |= (1 << TX_LED);
	// -------------------------------
	
	// ----- znicializujeme --------------
	 UCSR0B = 0x00;
	 UCSR0A = 0x02;
	 UCSR0C = 0x06;
	 UBRR0L = (uint8_t)BAUD_PRESCALE;
	 UBRR0H = (uint8_t)(BAUD_PRESCALE>>8);
	 UCSR0B = (1<<TXEN0)|(1<<TXCIE0)|(1<<RXEN0)|(1<<RXCIE0);
	//------------------------------------
	 stdout = &myStdOut; // !!!zatm nevyuit!!!
	 sei();	// povolme peruen

	// ---- pedme odkazy parametrum
	 volatile struct uart_s *p = &uart;
	 *packet = p->rx.packet;
}

// reciever interrupt handler
ISR (USART0_RX_vect)
{
	char c;
	static uint8_t start_pos = 0;
    volatile struct uart_s *p = &uart;
	
	RX_LED_ON;
	rx_led_blik = TRUE;
	
	if ((UCSR0A&((1<<FE0)|(1<<DOR0)|(1<<UPE0))))	// dolo-li k chyb
	// na plonku bude 2 barevn led pro indikaci komunikace, zelen pjem, erven vysln, blikn erven bude znait chybu komunikace
    {
        c = UDR0;
        return;
    }
    else
    {
		c = UDR0;
		if(c == '_') 
		{ 
			if( p->rx.packet[0] == NULL && p->rx.packet[1] == NULL && p->rx.packet[2] == NULL &&
			    p->rx.packet[3] == NULL && p->rx.packet[4] == NULL ) 
			{	   
			    p->rx.write_idx = 0;
			    p->rx.pkt_idx = 0;
			}
			
			start_pos = p->rx.write_idx;
			p->rx.dataontheway = TRUE; 
			return;
		}
        
		if(p->rx.dataontheway)
		{
			if (p->rx.write_idx < (RXBUFFLEN-1))
			{
				if (c != '!') 
				{
					p->rx.buffer[p->rx.write_idx++] = c;
				}
				else
				{
					p->rx.dataontheway = FALSE;
					p->rx.buffer[p->rx.write_idx++] = '\0';
					p->rx.packet[p->rx.pkt_idx++] = &(p->rx.buffer[start_pos]);
					
					if (p->rx.pkt_idx >= 5) 
					{ // no space, pokud je to 5 zprva je nutn vyvolat udlost peten zprv
					  // todo: takto zdlouhav innost tu samozejm nem co dlat e
						//uart_putstr_f(PSTR("FULL PACKET"));
					}	
				}
			}
			else
			{   // todo: takto zdlouhav innost tu samozejm nem co dlat e
				//uart_putstr_f(PSTR("FULL BUFFER"));// dolo v bufferu msto
				p->rx.dataontheway = FALSE; 
			}
		}
    }
}

// transmiter interrupt handler
ISR (USART0_TX_vect)
{
	TX_LED_ON;
	tx_led_blik = TRUE;
	
	if (uart.tx.left2send == 0 )
    {
		uart.tx.dataontheway = FALSE;
	}
	else
	{
        uint8_t nextidx = uart.tx.send_idx +1 ;
        if (nextidx >= TXBUFFLEN) nextidx = 0;
        UDR0 = uart.tx.buffer[uart.tx.send_idx];
        uart.tx.send_idx = nextidx;
		uart.tx.left2send--;
	}
}
